class InvalidGrantException(Exception):
    pass


class InvalidCredentialsException(Exception):
    pass


class ClientError(Exception):
    pass


class ServerError(Exception):
    pass


class HTTP400BadRequest(ClientError):
    pass


class HTTP401Unauthorized(ClientError):
    pass


class HTTP402PaymentRequired(ClientError):
    pass


class HTTP403Forbidden(ClientError):
    pass


class HTTP404NotFound(ClientError):
    pass


class HTTP405MethodNotAllowed(ClientError):
    pass


class HTTP406NotAcceptable(ClientError):
    pass


class HTTP407TemporaryRedirect(ClientError):
    pass


class HTTP408RequestTimeout(ClientError):
    pass


class HTTP409Conflict(ClientError):
    pass


class HTTP410Gone(ClientError):
    pass


class HTTP411LengthRequired(ClientError):
    pass


class HTTP412PreconditionFailed(ClientError):
    pass


class HTTP413PayloadTooLarge(ClientError):
    pass


class HTTP414UriTooLong(ClientError):
    pass


class HTTP415UnsupportedMediaType(ClientError):
    pass


class HTTP416RangeNotSatisfiable(ClientError):
    pass


class HTTP417ExpectationFailed(ClientError):
    pass


class HTTP418ImATeapot(ClientError):
    pass


class HTTP421MisdirectedRequest(ClientError):
    pass


class HTTP422UnprocessableEntity(ClientError):
    pass


class HTTP423Locked(ClientError):
    pass


class HTTP424FailedDependency(ClientError):
    pass


class HTTP425TooEarly(ClientError):
    pass


class HTTP426UpgradeRequired(ClientError):
    pass


class HTTP428PreconditionRequired(ClientError):
    pass


class HTTP429TooManyRequests(ClientError):
    pass


class HTTP431RequestHeaderFieldsTooLarge(ClientError):
    pass


class HTTP451UnavailableForLegalReasons(ClientError):
    pass


class HTTP500InternalServerError(ServerError):
    pass


class HTTP501NotImplemented(ServerError):
    pass


class HTTP502BadGateway(ServerError):
    pass


class HTTP503ServiceUnavailable(ServerError):
    pass


class HTTP504GatewayTimeout(ServerError):
    pass


class HTTP505HttpVersionNotSupported(ServerError):
    pass


class HTTP506VariantAlsoNegotiates(ServerError):
    pass


class HTTP507InsufficientStorage(ServerError):
    pass


class HTTP508LoopDetected(ServerError):
    pass


class HTTP510NotExtended(ServerError):
    pass


class HTTP511NetworkAuthenticationRequired(ServerError):
    pass


def raise_exception(status_code: int):
    if status_code // 100 != 4 and status_code // 100 != 5:
        return
    if status_code == 400:
        raise HTTP400BadRequest("The server couldn't resolve the request because of invalid syntax.")

    if status_code == 401:
        raise HTTP401Unauthorized("The client should authenticate itself to get the requested response.")

    if status_code == 402:
        raise HTTP402PaymentRequired(
            "Reserved for future use. The aim is to recommend the user to perform "
            "payment steps on digital payment systems."
        )

    if status_code == 403:
        raise HTTP403Forbidden(
            "The client does not have the access right for the content. "
            "Unlike 401, the identity of the client is known by the server."
        )

    if status_code == 404:
        raise HTTP404NotFound("The server can not find the requested resource.")

    if status_code == 405:
        raise HTTP405MethodNotAllowed(
            "The request method is known by the server but it is not supported by the target resource.")

    if status_code == 406:
        raise HTTP406NotAcceptable(
            "This response is sent when the web server, after performing server-driven content negotiation, "
            "doesn't find any content that conforms to the criteria given by the user agent."
        )

    if status_code == 407:
        raise HTTP407TemporaryRedirect(
            "Similar to 401 response but this time authentication is needed to be done by proxy.")

    if status_code == 408:
        raise HTTP408RequestTimeout(
            "This indicates the server didn't receive a complete request message within "
            "the server’s allotted timeout period."
        )

    if status_code == 409:
        raise HTTP409Conflict("This response is sent when a request conflicts with the current state of the server.")

    if status_code == 410:
        raise HTTP410Gone(
            "This is sent when the requested content has been "
            "permanently deleted from server, with no forwarding address."
        )

    if status_code == 411:
        raise HTTP411LengthRequired(
            "Server rejected the request because the Content-Length header field is not defined in the request.")

    if status_code == 412:
        raise HTTP412PreconditionFailed("Pre-defined preconditions in the header part didn't meet by the server.")

    if status_code == 413:
        raise HTTP413PayloadTooLarge("Request entity is larger than the limits defined by the server.")

    if status_code == 414:
        raise HTTP414UriTooLong("The URI requested by the client is longer than the server is willing to interpret.")

    if status_code == 415:
        raise HTTP415UnsupportedMediaType("The media format of the data is not supported.")

    if status_code == 416:
        raise HTTP416RangeNotSatisfiable(
            "The range specified by the Range header field in the request cannot be fulfilled.")

    if status_code == 417:
        raise HTTP417ExpectationFailed(
            "It means the expectation indicated by the Expect request header field cannot be met by server.")

    if status_code == 418:
        raise HTTP418ImATeapot(
            "This HTTP status is used as an Easter egg in some websites, including Google.com. "
            "The server refuses the attempt to brew coffee with a teapot."
        )

    if status_code == 421:
        raise HTTP421MisdirectedRequest("Request was directed to a server that is not able to create a response.")

    if status_code == 422:
        raise HTTP422UnprocessableEntity(
            "The request was well-formed but was unable to be followed due to semantic errors.")

    if status_code == 423:
        raise HTTP423Locked("The resource that is being accessed is locked for security reasons.")

    if status_code == 424:
        raise HTTP424FailedDependency("Request is failed due to failure of a previous request.")

    if status_code == 425:
        raise HTTP425TooEarly(
            "Indicates that the server is unwilling to risk processing a request that might be replayed.")

    if status_code == 426:
        raise HTTP426UpgradeRequired(
            "The server refuses to perform the request using the current protocol but might be willing to do so "
            "after the client upgrades to a different protocol."
        )

    if status_code == 428:
        raise HTTP428PreconditionRequired("The origin server requires the request to be conditional.")

    if status_code == 429:
        raise HTTP429TooManyRequests("The client has sent too many requests in a given time.")

    if status_code == 431:
        raise HTTP431RequestHeaderFieldsTooLarge(
            "Indicates that the header fields of the request are too large to handle.")

    if status_code == 451:
        raise HTTP451UnavailableForLegalReasons(
            "The user agent requested a resource that cannot legally be provided, "
            "such as a web page censored by a government."
        )

    if status_code == 500:
        raise HTTP500InternalServerError(
            "Server has faced an erroneous situation and it does not know how to handle that.")

    if status_code == 501:
        raise HTTP501NotImplemented("The request method is not supported by the server.")

    if status_code == 502:
        raise HTTP502BadGateway(
            "This error response indicates that the server, "
            "while working as a gateway to get a response needed to handle the request, "
            "got an invalid response during the flow."
        )

    if status_code == 503:
        raise HTTP503ServiceUnavailable(
            "The server is not ready to handle the request yet. "
            "It can be because the server is down for maintenance or it is overloaded."
        )

    if status_code == 504:
        raise HTTP504GatewayTimeout("The server is acting like a gateway and it cannot get a response in a given time.")

    if status_code == 505:
        raise HTTP505HttpVersionNotSupported(
            "The HTTP version used in the request is not supported by that specific server.")

    if status_code == 506:
        raise HTTP506VariantAlsoNegotiates(
            "the chosen variant resource is configured to engage in transparent content negotiation itself, "
            "and is therefore not a proper end point in the negotiation process."
        )

    if status_code == 507:
        raise HTTP507InsufficientStorage(
            "The method could not be performed on the resource because the server is unable to store the "
            "representation needed to successfully complete the request."
        )

    if status_code == 508:
        raise HTTP508LoopDetected("The server detected an infinite loop while processing the request.")

    if status_code == 510:
        raise HTTP510NotExtended("Further extensions to the request are required for the server to fulfill it.")

    if status_code == 511:
        raise HTTP511NetworkAuthenticationRequired("The client needs to authenticate itself to gain network access.")



import unittest
import sys
import json

sys.path.append('..')

from FCSAPIAccess import *

with open("../credentials.json") as f:
    credentials = json.load(f)


class TestCaseComparisons(unittest.TestCase):
    def test_correct_credentials(self):
        FCSAPIAccess(
            credentials["client_id"], credentials["client_secret"],
            []
        )

    def test_expired_access_token(self):
        access = FCSAPIAccess(
            credentials["client_id"], credentials["client_secret"],
            Scope.FULL_ACCESS
        )

        access.set_access_token(credentials["expired_access"], credentials["expired_refresh"])
        self.assertNotEqual(
            access.project.list_directories(),
            {'error': 'expired_code', 'error_description': 'The provided code is expired.'}
        )

    def test_incorrect_credentials(self):
        self.assertRaises(
            InvalidGrantException,
            lambda: FCSAPIAccess(
                credentials["client_id"] + "BAD", credentials["client_secret"],
                Scope.FULL_ACCESS
            )
        )


if __name__ == '__main__':
    unittest.main()



"""
WARNING: THIS MODULE IS GENERATED. CODE CHANGES WILL BE DESTROYED!
"""
import enum


class Scope(enum.Enum):
    FULL_ACCESS = "*:*:*"
    USER_VALIDATE_TOKEN = "user:validate:token"
    USER_VIEW_DATA = "user:view:data"
    USER_VIEW_EMAIL = "user:view:email"
    USER_UPDATE_DATA = "user:update:data"
    USER_UPDATE_EMAIL = "user:update:email"
    PROJECT_UPDATE_API_ACCESS = "project:update:api_access"
    PROJECT_UPDATE_USER_APPLICATION = "project:update:user_application"
    PROJECT_UPDATE_AUTO_AUTH = "project:update:auto_auth"
    PROJECT_UPDATE_FIELDS = "project:update:fields"
    PROJECT_UPDATE_LIMIT = "project:update:limit"
    PROJECT_UPDATE_PASSWORD = "project:update:password"
    PROJECT_UPDATE_ROLES = "project:update:roles"
    PROJECT_UPDATE_SCOPE = "project:update:scope"
    PROJECT_UPDATE_SETTINGS = "project:update:settings"
    PROJECT_UPDATE_THIRD_PARTY = "project:update:third_party"
    PROJECT_UPDATE_USER = "project:update:user"
    PROJECT_UPDATE_VALIDATION = "project:update:validation"
    PROJECT_VIEW_API_ACCESS = "project:view:api_access"
    PROJECT_VIEW_APPLICATION = "project:view:application"
    PROJECT_VIEW_AUTO_AUTH = "project:view:auto_auth"
    PROJECT_VIEW_FIELDS = "project:view:fields"
    PROJECT_VIEW_LIMIT = "project:view:limit"
    PROJECT_VIEW_PASSWORD = "project:view:password"
    PROJECT_VIEW_ROLES = "project:view:roles"
    PROJECT_VIEW_SCOPE = "project:view:scope"
    PROJECT_VIEW_SETTINGS = "project:view:settings"
    PROJECT_VIEW_THIRD_PARTY = "project:view:third_party"
    PROJECT_VIEW_USER = "project:view:user"
    PROJECT_VIEW_VALIDATION = "project:view:validation"
    PROJECT_VIEW_REDIRECT_URI = "project:view:redirect_uri"
    PROJECT_UPDATE_REDIRECT_URI = "project:update:redirect_uri"
    MONITOR_VIEW_URL = "monitor:view:url"
    MONITOR_UPDATE_URL = "monitor:update:url"
    MONITOR_VIEW_STATE = "monitor:view:state"
    NOTIFICATION_VIEW_CHANNEL = "notification:view:channel"
    NOTIFICATION_UPDATE_CHANNEL = "notification:update:channel"
    NOTIFICATION_PULL_MESSAGE = "notification:pull:message"
    NOTIFICATION_PUSH_MESSAGE = "notification:push:message"
    NOTIFICATION_VIEW_CHANNEL_RECIPIENT = "notification:view:channel_recipient"
    NOTIFICATION_UPDATE_CHANNEL_RECIPIENT = "notification:update:channel_recipient"
    NOTIFICATION_VIEW_HISTORY = "notification:view:history"
    NOTIFICATION_VIEW_STATISTICS = "notification:view:statistics"
    PROJECT_VIEW_RECIPIENT = "project:view:recipient"
    PROJECT_UPDATE_RECIPIENT = "project:update:recipient"
    TRANSLATION_ENQUEUE_TRANSLATION = "translation:enqueue:translation"
    TRANSLATION_POLL_TRANSLATION = "translation:poll:translation"
    TRANSLATION_DOWNLOAD_TRANSLATION = "translation:download:translation"
    TRANSLATION_VIEW_LANGUAGES = "translation:view:languages"
    TRANSLATION_VIEW_FORMAT = "translation:view:format"
    ENTITY_UPDATE_API = "entity:update:api"
    ENTITY_VIEW_API = "entity:view:api"
    ENTITY_UPDATE_VERSION = "entity:update:version"
    ENTITY_VIEW_VERSION = "entity:view:version"
    ENTITY_UPDATE_PERMISSION = "entity:update:permission"
    ENTITY_VIEW_PERMISSION = "entity:view:permission"
    ENTITY_UPDATE_DATA = "entity:update:data"
    ENTITY_VIEW_DATA = "entity:view:data"
    ENTITY_UPDATE_STRUCTURE = "entity:update:structure"
    ENTITY_VIEW_STRUCTURE = "entity:view:structure"
    TRANSLATION_ENQUEUE_EXTRACT = "translation:enqueue:extract"
    TRANSLATION_ENQUEUE_LIBRARY = "translation:enqueue:library"
    PROJECT_VIEW_EMAIL_CATEGORY = "project:view:email_category"
    PROJECT_UPDATE_EMAIL_CATEGORY = "project:update:email_category"
    USER_VIEW_EMAIL_SUBSCRIPTIONS = "user:view:email_subscriptions"
    USER_UPDATE_EMAIL_SUBSCRIPTIONS = "user:update:email_subscriptions"
    PROJECT_VIEW_MAILING_LIST = "project:view:mailing_list"
    USER_CREATE_NEW = "user:create:new"
    COMPANY_VIEW_ADMIN = "company:view:admin"
    COMPANY_UPDATE_ADMIN = "company:update:admin"
    COMPANY_VIEW_PERMISSIONS = "company:view:permissions"
    COMPANY_UPDATE_PERMISSIONS = "company:update:permissions"
    COMPANY_VIEW_INFO = "company:view:info"
    COMPANY_UPDATE_INFO = "company:update:info"
    APPLICATION_VIEW_SESSION = "application:view:session"
    APPLICATION_EXPIRE_SESSION = "application:expire:session"
    APPLICATION_REFRESH_SESSION = "application:refresh:session"
    APPLICATION_REVOKE_SESSION = "application:revoke:session"
    APPLICATION_VIEW_SCOPE = "application:view:scope"
    APPLICATION_UPDATE_SCOPE = "application:update:scope"
    USER_APPLICATION_VIEW_SCOPE = "user_application:view:scope"
    USER_APPLICATION_UPDATE_SCOPE = "user_application:update:scope"
    FILE_VIEW_PROJECT = "file:view:project"
    FILE_VIEW_USER = "file:view:user"
    FILE_UPDATE_USER = "file:update:user"
    FILE_VIEW_FILE = "file:view:file"
    FILE_UPDATE_FILE = "file:update:file"
    COMPANY_VIEW_PROJECT = "company:view:project"
    PROJECT_IMPORT_SETTINGS = "project:import:settings"



"""
WARNING: THIS MODULE IS GENERATED! ALL CHANGES WILL BE DISCARDED
"""

import requests
import urllib.parse


class FangTranslationServices:
    """
    Welcome to the FangTranslationServices API!
    
    This API is for translating files for automated application localization
    """

    url_base = "https://fangcloudservices.pythonanywhere.com/api/v1"

    def __init__(self, access_token):
        self.headers = {'Authorization': 'Bearer {}'.format(access_token)}
        self._status_check = lambda x, y: None

    def _check_status(self, r: requests.Response, retry: callable):
        check = self._status_check(r, retry)
        
        if "json" in r.headers['content-type']:
            result = r.json()
            
        else:
            result = r.text
        
        return result if check is None else check
        
    def _url_encode(self, text: str) -> str:
        return urllib.parse.quote_plus(str(text))
    
    def poll_translation(self, id) -> dict:
        """
        Polls the status of an ongoing translation job.
        
        When a translation is complete, the value of `completed` will change from `null` to the datetime of completion.
        
        See the *Download Translation* section to download the processed file
        
        Translations are removed 24 hours after creation.
        :param id: The ID of the translation job
        """
        local_vars = locals()
        if 'self' in local_vars: del local_vars['self']

        headers = self.headers.copy()
        r = requests.request(
            "GET", 
            "https://fangcloudservices.pythonanywhere.com/api/v1/translation?id={}".format(self._url_encode(id)), 
            headers=headers
        )
        
        return self._check_status(r, lambda: self.poll_translation(**local_vars))
        
    def enqueue_translation(self) -> dict:
        """
        Adds a file to the translation queue to be translated.
        
        Files can only be translated from English at this time.
        
        NOTE: Translations may not be 100% accurate.
        """
        local_vars = locals()
        if 'self' in local_vars: del local_vars['self']

        raise NotImplementedError('The requested endpoint is not ready for use')

    def download_translation(self, id) -> dict:
        """
        Allows the specified translation job to be downloaded after completion.
        
        If this endpoint is called and the job is not yet complete, you will receive an HTTP: 425 response code.
        :param id: The ID of the translation job
        """
        local_vars = locals()
        if 'self' in local_vars: del local_vars['self']

        headers = self.headers.copy()
        r = requests.request(
            "GET", 
            "https://fangcloudservices.pythonanywhere.com/api/v1/translation/download?id={}".format(self._url_encode(id)), 
            headers=headers
        )
        
        return self._check_status(r, lambda: self.download_translation(**local_vars))
        
    def get_languages(self) -> dict:
        """
        Retrieves the list of supported languages
        """
        local_vars = locals()
        if 'self' in local_vars: del local_vars['self']

        headers = self.headers.copy()
        r = requests.request(
            "GET", 
            "https://fangcloudservices.pythonanywhere.com/api/v1/translation/languages", 
            headers=headers
        )
        
        return self._check_status(r, lambda: self.get_languages(**local_vars))
        
    def get_formats(self) -> dict:
        """
        Retrieves the list of supported file formats
        """
        local_vars = locals()
        if 'self' in local_vars: del local_vars['self']

        headers = self.headers.copy()
        r = requests.request(
            "GET", 
            "https://fangcloudservices.pythonanywhere.com/api/v1/translation/formats", 
            headers=headers
        )
        
        return self._check_status(r, lambda: self.get_formats(**local_vars))
        
    def extract_text(self, format) -> dict:
        """
        Converts the provided document to a JSON translation file which is compatable with the translator. The contents of the document are sent as the body of the request.
        
        **Note**: While we do our best to create descriptive keys, to reference the translations, the system is by no means perfect. Key names may be very bulky/bloated and some text may be missed. We strongly advise a manual review of the file before using it anywhere else.
        
        Supported input file formats:
        
        - HTML
            
        
        _More coming soon_
        :param format: The format of the input document
        """
        local_vars = locals()
        if 'self' in local_vars: del local_vars['self']

        headers = self.headers.copy()
        r = requests.request(
            "POST", 
            "https://fangcloudservices.pythonanywhere.com/api/v1/translation/extract?format={}".format(self._url_encode(format)), 
            headers=headers
        )
        
        return self._check_status(r, lambda: self.extract_text(**local_vars))
        
    def generate_library(self, language) -> dict:
        """
        Generates a new file from the provided JSON translation file which can be imported into your project to make translation management easier.
        
        Please refer to the _Library Generator_ page under _Translations_ when logged into your account for information on each library.
        
        Supported languages:
        
        - JavaScript
            
        
        _More coming soon_
        :param language: The language to generate the library for
        """
        local_vars = locals()
        if 'self' in local_vars: del local_vars['self']

        raise NotImplementedError('The requested endpoint is not ready for use')
